﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Text;
using System.Threading;
using Newtonsoft.Json;

namespace data_sync_client
{
    class PullService
    {
        private  TokenService tokenService;
        private  DataHandler dataHandler;

        public PullService(TokenService tokenService, DataHandler dataHandler)
        {
            this.tokenService = tokenService;
            this.dataHandler = dataHandler;
        }

        public void start()
        {
            while (true)
            {

                int sleepTime = 0;
                try
                {
                    HttpWebRequest request = (HttpWebRequest)WebRequest.Create("https://dev.thuwater.com/api/data/realtime");
                    request.Method = "GET";
                    request.Headers.Set("X-TH-TOKEN", tokenService.genToken());
                    request.Host = "dev.thuwater.com";
                    HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                    if (response.StatusCode == HttpStatusCode.OK)
                    {
                        Stream myResponseStream = response.GetResponseStream();
                        StreamReader myStreamReader = new StreamReader(myResponseStream, Encoding.GetEncoding("utf-8"));
                        string resBody = myStreamReader.ReadToEnd();
                        myStreamReader.Close();
                        myResponseStream.Close();
                        if (string.IsNullOrEmpty(resBody))
                        {
                            sleepTime = 5 * 60 * 1000;
                            continue;
                        }
                        List<ThData> thDataList = JsonConvert.DeserializeObject<List<ThData>>(resBody);
                        if(thDataList.Count == 0)
                        {
                            sleepTime = 5 * 60 * 1000;
                        }
                        else if(thDataList.Count < 100)
                        {
                            dataHandler.handleDatas(thDataList);
                            // 当返回数据少于100条的时候，建议等待1分钟后再请求
                            sleepTime = 60 * 1000;
                        }
                        else
                        {
                            // 当返回数据大于100条的时候，建议不等待直接继续请求最新数据
                            dataHandler.handleDatas(thDataList);
                        }
                    }
                    else
                    {
                        //  log.error("请求返回错误，请检查[{}]", response);
                        sleepTime = 60 * 1000;
                        if (response.StatusCode == HttpStatusCode.Unauthorized)
                        {
                            tokenService.genToken(true);
                        }
                    }

                } catch (IOException e)
                {
                    NetLog.WriteTextLog("err",e.Message);
                    sleepTime = 60 * 1000;// 过1分钟后再重试
                }
                catch (Exception e)
                {
                    NetLog.WriteTextLog("err", e.Message);
                    sleepTime = 60 * 1000;// 过1分钟后再重试
                }
                finally
                {
                    sleep(sleepTime);
                }
            }
        }
        private void sleep(int time)
        {
            NetLog.WriteTextLog("info", "sleep for " + time);
            Thread.Sleep(time);
        }
    }
}
